==> ad-5.x-1.0-1.tar.gz: ad/adserve.php <==
<?php
// $Id: adserve.php,v 1.2.2.15.2.8 2007/05/03 16:11:26 jeremy Exp $

adserve();

/**
 * The main adserve logic.
 */
function adserve() {
  adserve_variable('ad_dir', getcwd());

  // Move process to top level Drupal directory so we can perform a bootstrap.
  for ($i = 1; $i <= 20; $i++) {
    if (file_exists('./includes/bootstrap.inc')) {
      break;
    }
    chdir('..');
  }
  adserve_variable('root_dir', getcwd());
  require_once adserve_variable('root_dir') .'/includes/bootstrap.inc';

  adserve_debug();

  $error = FALSE;
  $output = NULL;
  if (adserve_variable('cache') != 'none') {
    /**
     * Ad caches are defined through external modules.  Ad caches are composed 
     * of a module 'ad_cache_TYPE.module' and an include file 
     * 'ad_cache_TYPE.inc' that live in the 'cache/TYPE' subdirectory where 
     * 'TYPE' is replaced with the type of cache.  For example, the included 
     * file cache lives in 'cache/file'.
     *
     * The ad_cache_TYPE.inc file must have a function named ad_cache_TYPE()
     * which is used to display ads.  It can optionally include a function
     * titled ad_cache_TYPE_variables used to extract any necessary
     * variables from the global $_GET array (this can also be used to override
     * values that would normally be set from $_GET).  Any functions used
     * by this code without bootstrapping Drupal should also be in this file.
     *
     * The ad_cache_TYPE.module file should define the drupal _help() hook
     * so the module can be enabled.  It should also define the _adcacheapi()
     * hook allowing for configuration and processing.  Any functions used by
     * this code after bootstrapping Drupal should also be in this module.
     *
     * Refer to cache/file/* for an implementation example.
     */
    $function = 'ad_cache_'. adserve_variable('cache');
    if (function_exists("$function")) {
      $output = $function();
    }
    else if (adserve_variable('debug')) {
      echo "Function '$function' does not exist.<br />\n";
    }
  }

  // If cache fails above (or if no cache is specified) we don't use a cache.
  if ($output == NULL) {
    if (adserve_variable('debug')) {
      echo "No cache enabled.<br />\n";
    }
  
    adserve_bootstrap();
    
    if (adserve_variable('nids')) {
      $nids = explode(',', adserve_variable('nids'));
  
      // Build query to filter on just selected nids.
      $filter = '';
      foreach ($nids as $nid) {
        if ($filter) {
          $filter .= " OR aid = $nid";
        }
        else {
          $filter = " AND (aid = $nid";
        }
      }
      $filter .= ')';
  
      // Randomly determine which ad to display from the specified nids.
      $result = db_query("SELECT aid, adtype, redirect, activated, maxviews, maxclicks FROM {ads} a WHERE adstatus = 'active' $filter ORDER BY RAND() LIMIT %d", adserve_variable('quantity'));
    }
    else {
      // Randomly determine which ad to display from the specified group.
      $result = db_query("SELECT a.aid, a.adtype, a.redirect, a.activated, a.maxviews, a.maxclicks FROM {ads} a INNER JOIN {ad_groups} g ON a.gid = g.gid WHERE a.adstatus = 'active' AND g.name = '%s' ORDER BY RAND() LIMIT %d", adserve_variable('group'), adserve_variable('quantity'));
    }
      
    $output = '';
    $ads = 0;
    // Include appropriate module for displaying selected ad.
    while ($ad = db_fetch_object($result)) {
      $ads++;
      $module = db_result(db_query("SELECT filename FROM {system} WHERE name = '%s'", "ad_$ad->adtype"));
      if (adserve_variable('debug')) {
        echo "ad: <pre>";
        print_r($ad);
        echo "</pre>";
        echo "Loading module '$module'.<br />\n";
      }
      include_once "$module";
      
      if ($output) {
        // TODO: Make this configurable or themeable.
        $output .= '<br />';
      }
      $output .= module_invoke("ad_$ad->adtype", 'display_ad', $ad, adserve_variable('hostid'));
  
      // Update view statistics.
      db_query("UPDATE {ad_statistics} SET count = count + 1 WHERE aid = %d AND action = 'view' AND date = %d AND hostid = '%s'", $ad->aid, date('YmdH'), adserve_variable('hostid'));
      // If column doesn't already exist, we need to add it.
      if (!db_affected_rows()) {
        db_query("INSERT INTO {ad_statistics} (aid, date, action, hostid, count) VALUES(%d, %d, 'view', '%s', 1)", $ad->aid, date('YmdH'), adserve_variable('hostid'));
        // If another process already added this row our INSERT will fail, if 
        // so we still need to increment it so we don't loose a view.
        if (!db_affected_rows()) {
          db_query("UPDATE {ad_statistics} SET count = count + 1 WHERE aid = %d AND action = 'view' AND date = %d AND hostid = '%s'", $ad->aid, date('YmdH'), adserve_variable('hostid'));
        }
      }
  
      // See if we need to perform additional queries.
      if ($ad->maxviews) {
        $views = (int)db_result(db_query("SELECT SUM(count) FROM {ad_statistics} WHERE aid = %d AND action = 'view' AND date >= %d", $ad->aid, date('YmdH', $ad->activated)));
        if ($views >= $ad->maxviews) {
          db_query("UPDATE {ads} SET adstatus = 'expired', autoexpire = 0, autoexpired = %d, expired = %d WHERE aid = %d", time(), time(), $ad->aid);
          ad_statistics_increment($ad->aid, 'autoexpired');
          ad_statistics_increment($ad->aid, 'expired');
        }
      }
      if ($ad->maxclicks) {
        $clicks = (int)db_result(db_query("SELECT SUM(count) FROM {ad_statistics} WHERE aid = %d AND action = 'click' AND date >= %d", $ad->aid, date('YmdH', $ad->activated)));
        if ($clicks >= $ad->maxclicks) {
          db_query("UPDATE {ads} SET adstatus = 'expired', autoexpire = 0, autoexpired = %d, expired = %d WHERE aid = %d", time(), time(), $ad->aid);
          ad_statistics_increment($ad->aid, 'autoexpired');
          ad_statistics_increment($ad->aid, 'expired');
        }
      }
    }
    if (empty($ads)) {
      $error = TRUE;
      if (!empty($nids)) {
        $output = 'No active ads were found in the nids "'. adserve_variable('nids') .'"';
      }
      else {
        $output = 'No active ads were found in the group "'. adserve_variable('group') .'"';
      }
    }
    if (adserve_variable('debug')) {
      echo "Ads displayed: $ads<br />";
    }
  }

  if (!adserve_variable('debug')) {
    header("Content-Type: application/x-javascript; charset=utf-8");
  }
  print "document.write('$output');";
  /**
   * TODO:
   * At some point we probably don't want to display errors to the screen, as
   * certain filters may intentionally not display ads.  But for now it's useful
   * in debugging.
  if ($error) {
    print "document.write('<!-- $output -->');";
  }
  else {
    print "document.write('$output');";
  }
  */
  exit(0);
}


/**
 * Retrieve variables from $_GET array.
 */
function adserve_variable($variable, $value = NULL) {
  static $variables = NULL;

  if (isset($value)) {
    $variables->$variable = $value;
  }

  if (!isset($variables->loaded)) {
    // 'debug' is an integer.
    $variables->debug = isset($_GET['debug']) ? (int)$_GET['debug'] : 0;
    // Group names are comprised of only alphanumerics.
    $variables->group = isset($_GET['g']) ? preg_replace('/[^0-9a-zA-Z]/', '', $_GET['g']) : 'default';
    // Cache types are comprised of only letters.
    $variables->cache = isset($_GET['c']) ? preg_replace('/[^a-zA-Z]/', '', $_GET['c']) : 'none';
    // Nids is an integer or a ",".
    $variables->nids = isset($_GET['n']) ? preg_replace('/[^0-9,]/', '', $_GET['n']) : '';
    // Hostid is an md5() which is comprised of numbers and letters a-f.
    $variables->hostid = isset($_GET['k']) ? preg_replace('/[^0-9a-f]/', '', $_GET['k']) : '';

    // Quantity is an integer.
    $variables->quantity = isset($_GET['q']) ? (int)$_GET['q'] : 0;

    // Retrieve variables defined by cache plugin, if enabled.
    if ($variables->cache != 'none') {
      $include = $variables->ad_dir ."/cache/$variables->cache/ad_cache_$variables->cache.inc";
      if (file_exists($include)) {
        if ($variables->debug) {
          echo "Attempting to include cache include file '$include'.<br />\n";
        }
        require_once ($include);
      }
      else if ($variables->debug) {
        echo "Failed to find cache include file '$include'.<br />\n";
      }
      $function = "ad_cache_$variables->cache". '_variables';
      if (function_exists($function)) {
        $external_variables = $function();
        foreach ($external_variables as $key => $value) {
          $variables->$key = $value;
        }
      }
    }

    // Set defaults.
    $variables->quantity = $variables->quantity ? $variables->quantity : 1;

    if ($variables->debug) {
      foreach ($variables as $variable => $value) {
        echo "$variable: '$value'<br />\n";
      }
      if ($variables->debug == 1) exit;
    }
    $variables->loaded = TRUE;
  }

  return $variables->$variable;
}

/**
 * Display additional debug information.
 */
function adserve_debug() {
  if (adserve_variable('debug')) {
    echo "Root drupal directory detected as '". adserve_variable('root_dir') ."'.<br />\n<br />\n";

    $ad_dir = adserve_variable('ad_dir');
    $files = array("$ad_dir/adserve.php", "$ad_dir/ad.module");
    if (adserve_variable('debug') > 2) {
      $files = array_merge($files, array("$ad_dir/ad.install", "$ad_dir/ad_views.inc"));
    }
    if (adserve_variable('debug') > 3) {
      $files = array_merge($files, array("$ad_dir/image/ad_image.module", "$ad_dir/image/ad_image.install", "$ad_dir/image/ad_image_views.inc", "$ad_dir/text/ad_text.module", "$ad_dir/text/ad_text.install", "$ad_dir/text/ad_text_views.inc", "$ad_dir/embed/ad_embed.module", "$ad_dir/report/ad_report.module", "$ad_dir/notify/ad_notify.module", "$ad_dir/notify/ad_notify.install"));
    }
    foreach ($files as $file) {
      if (!file_exists($file)) {
        echo "Error: '$file' does not exist!<br />\n";
      }
      else if (!is_readable($file)) {
        echo "Error: '$file' is not readable!<br />\n";
      }
      else {
        $fd = fopen($file, 'r');
        while (!feof($fd)) {
          $line = fgets($fd);
          if (substr($line, 0, 5) == "<?php") {
            continue;
          }
          else {
            echo "$file: $line<br />";
            break;
          }
        }
      }
    }
    echo "<br />\n";
  }
}

/**
 * Include the necessary files and call the Drupal bootstrap.
 */
function adserve_bootstrap($bootstrap = DRUPAL_BOOTSTRAP_FULL) {
  if (adserve_variable('debug')) {
    echo "Drupal bootstrap '". $bootstrap ."'.<br />\n";
  }

  drupal_bootstrap($bootstrap);
}

?>

==> ad-5.x-1.1-rc1.tar.gz: ad/adserve.php <==
<?php
// $Id: adserve.php,v 1.2.2.15.2.17 2007/05/27 21:55:47 jeremy Exp $

adserve();

/**
 * The main adserve logic.
 */
function adserve() {
  static $displayed_count = 0;

  adserve_variable('ad_dir', getcwd());

  // Move process to top level Drupal directory so we can perform a bootstrap.
  for ($i = 1; $i <= 20; $i++) {
    if (file_exists('./includes/bootstrap.inc')) {
      break;
    }
    chdir('..');
  }
  adserve_variable('root_dir', getcwd());
  require_once adserve_variable('root_dir') .'/includes/bootstrap.inc';

  adserve_debug();

  adserve_variable('error', FALSE);
  $output = NULL;
  if (adserve_variable('cache') != 'none') {
    /**
     * Ad caches are defined through external modules.  Ad caches are composed 
     * of a module 'ad_cache_TYPE.module' and an include file 
     * 'ad_cache_TYPE.inc' that live in the 'cache/TYPE' subdirectory where 
     * 'TYPE' is replaced with the type of cache.  For example, the included 
     * file cache lives in 'cache/file'.
     *
     * The ad_cache_TYPE.inc file must have a function named ad_cache_TYPE()
     * which is used to display ads.  It can optionally include a function
     * titled ad_cache_TYPE_variables used to extract any necessary
     * variables from the global $_GET array (this can also be used to override
     * values that would normally be set from $_GET).  Any functions used
     * by this code without bootstrapping Drupal should also be in this file.
     *
     * The ad_cache_TYPE.module file should define the drupal _help() hook
     * so the module can be enabled.  It should also define the _adcacheapi()
     * hook allowing for configuration and processing.  Any functions used by
     * this code after bootstrapping Drupal should also be in this module.
     *
     * Refer to cache/file/* for an implementation example.
     */
    $function = 'ad_cache_'. adserve_variable('cache');
    if (function_exists("$function")) {
      $output = $function();
    }
    else if (adserve_variable('debug')) {
      echo "Function '$function' does not exist.<br />\n";
    }
  }

  // If cache fails above (or if no cache is specified) we don't use a cache.
  if ($output == NULL) {
    if (adserve_variable('debug')) {
      echo "No cache enabled.<br />\n";
    }
  
    adserve_bootstrap();
    
    if (adserve_variable('nids')) {
      $id = adserve_variable('nids');
      $nids = explode(',', $id);
      if (adserve_variable('debug')) {
        echo "Searching for ad from nid list: $id.<br />\n";
      }
  
      // Build query to filter on just selected nids.
      $filter = '';
      foreach ($nids as $nid) {
        if ($filter) {
          $filter .= " OR aid = $nid";
        }
        else {
          $filter = " AND (aid = $nid";
        }
      }
      $filter .= ')';
  
      // Randomly determine which ad to display from the specified nids.
      $result = db_query("SELECT aid, adtype, redirect, activated, maxviews, maxclicks FROM {ads} a WHERE adstatus = 'active' $filter ORDER BY RAND() LIMIT %d", adserve_variable('quantity'));
    }
    else if (adserve_variable('tids')) {
      $id = adserve_variable('tids');
      $tids = explode(',', $id);

      if (adserve_variable('debug')) {
        echo "Searching for ad from tid list: $id.<br />\n";
      }

      // Build query to filter on just selected tids.
      $filter = '';
      foreach ($tids as $tid) {
        if ($filter) {
          $filter .= " OR n.tid = $tid";
        }
        else {
          $filter = " AND (n.tid = $tid";
        }
      }
      $filter .= ')';
  
      // Randomly determine which ad to display from the specified tids.
      $result = db_query("SELECT a.aid, a.adtype, a.redirect, a.activated, a.maxviews, a.maxclicks FROM {ads} a INNER JOIN {term_node} n ON a.aid = n.nid WHERE a.adstatus = 'active' $filter ORDER BY RAND() LIMIT %d", adserve_variable('quantity'));
    }
    else {
      $id = 0;
      // Randomly determine which ad to display from those that do not have
      // any tid assigned to them.
      $result = db_query("SELECT a.aid, a.adtype, a.redirect, a.activated, a.maxviews, a.maxclicks FROM {ads} a INNER JOIN {term_node} n ON a.aid = n.nid WHERE a.adstatus = 'active' AND n.tid = 0 ORDER BY RAND() LIMIT %d", adserve_variable('quantity'));
    }
      
    $output = '';
    $ads = 0;
    // Include appropriate module for displaying selected ad.
    while ($ad = db_fetch_object($result)) {
      $ads++;
      $module = db_result(db_query("SELECT filename FROM {system} WHERE name = '%s'", "ad_$ad->adtype"));
      if (adserve_variable('debug')) {
        echo "ad: <pre>";
        print_r($ad);
        echo "</pre>";
        echo "Loading module '$module'.<br />\n";
      }
      include_once "$module";
      
      if ($output) {
        // Add a div between ads that themers can use to arrange ads when 
        // displaying more than one at a time.
        $displayed_count++;
        $output .= "<div class=\"advertisement-space\" id=\"$id-$displayed_count\"></div>";
      }
      $output .= module_invoke("ad_$ad->adtype", 'display_ad', $ad);
  
      // Update view statistics.
      db_query("UPDATE {ad_statistics} SET count = count + 1 WHERE aid = %d AND action = 'view' AND date = %d AND hostid = '%s'", $ad->aid, date('YmdH'), adserve_variable('hostid'));
      // If column doesn't already exist, we need to add it.
      if (!db_affected_rows()) {
        db_query("INSERT INTO {ad_statistics} (aid, date, action, hostid, count) VALUES(%d, %d, 'view', '%s', 1)", $ad->aid, date('YmdH'), adserve_variable('hostid'));
        // If another process already added this row our INSERT will fail, if 
        // so we still need to increment it so we don't loose a view.
        if (!db_affected_rows()) {
          db_query("UPDATE {ad_statistics} SET count = count + 1 WHERE aid = %d AND action = 'view' AND date = %d AND hostid = '%s'", $ad->aid, date('YmdH'), adserve_variable('hostid'));
        }
      }
  
      // See if we need to perform additional queries.
      if ($ad->maxviews) {
        $views = (int)db_result(db_query("SELECT SUM(count) FROM {ad_statistics} WHERE aid = %d AND action = 'view' AND date >= %d", $ad->aid, date('YmdH', $ad->activated)));
        if ($views >= $ad->maxviews) {
          db_query("UPDATE {ads} SET adstatus = 'expired', autoexpire = 0, autoexpired = %d, expired = %d WHERE aid = %d", time(), time(), $ad->aid);
          ad_statistics_increment($ad->aid, 'autoexpired');
          ad_statistics_increment($ad->aid, 'expired');
        }
      }
      if ($ad->maxclicks) {
        $clicks = (int)db_result(db_query("SELECT SUM(count) FROM {ad_statistics} WHERE aid = %d AND action = 'click' AND date >= %d", $ad->aid, date('YmdH', $ad->activated)));
        if ($clicks >= $ad->maxclicks) {
          db_query("UPDATE {ads} SET adstatus = 'expired', autoexpire = 0, autoexpired = %d, expired = %d WHERE aid = %d", time(), time(), $ad->aid);
          ad_statistics_increment($ad->aid, 'autoexpired');
          ad_statistics_increment($ad->aid, 'expired');
        }
      }
    }
    if (empty($ads)) {
      adserve_variable('error', TRUE);
      $output = 'No active ads were found in the '. (empty($nids) ? 'tids' : 'nids') ." '$id'.";
    }
    if (adserve_variable('debug')) {
      echo "Ads displayed: $ads<br />";
    }
  }

  if (!adserve_variable('debug')) {
    // Tell the web browser not to cache this script so the ad refreshes each
    // time the page is viewed.
    header('Expires: Mon, 26 Jul 1997 05:00:00 GMT'); // Expires in the past.
    header('Last-Modified: '. gmdate('D, d M Y H:i:s') .' GMT'); // Last load.
    header('Cache-Control: no-store, no-cache, must-revalidate'); // HTTP 1.1
    header('Cache-Control: post-check=0, pre-check=0', false);    // HTTP 1.1
    header('Pragma: no-cache'); // HTTP 1.0
    // Output is a JavaScript.
    header('Content-Type: application/x-javascript; charset=utf-8');
  }
  $output = preg_replace('&/@HOSTID___&', '/'. adserve_variable('hostid'), $output);
  if (adserve_variable('error')) {
    print "document.write('<!-- $output -->');";
  }
  else {
    print "document.write('$output');";
  }
  exit(0);
}


/**
 * Retrieve variables from $_GET array.
 */
function adserve_variable($variable, $value = NULL) {
  static $variables = NULL;

  if (isset($value)) {
    $variables->$variable = $value;
  }

  if (!isset($variables->loaded)) {
    // 'debug' is an integer.
    $variables->debug = isset($_GET['debug']) ? (int)$_GET['debug'] : 0;
    // Cache types are comprised of only letters.
    $variables->cache = isset($_GET['c']) ? preg_replace('/[^a-zA-Z]/', '', $_GET['c']) : 'none';
    // Nids is an integer or a ",".
    $variables->nids = isset($_GET['n']) ? preg_replace('/[^0-9,]/', '', $_GET['n']) : '';
    // Tids is an integer or a ",".
    $variables->tids = isset($_GET['t']) ? preg_replace('/[^0-9,]/', '', $_GET['t']) : '';
    // Hostid is an md5() which is comprised of numbers and letters a-f.
    $variables->hostid = isset($_GET['k']) ? preg_replace('/[^0-9a-f]/', '', $_GET['k']) : '';

    // Quantity is an integer.
    $variables->quantity = isset($_GET['q']) ? (int)$_GET['q'] : 0;

    // Retrieve variables defined by cache plugin, if enabled.
    if ($variables->cache != 'none') {
      $include = $variables->ad_dir ."/cache/$variables->cache/ad_cache_$variables->cache.inc";
      if (file_exists($include)) {
        if ($variables->debug) {
          echo "Attempting to include cache include file '$include'.<br />\n";
        }
        require_once ($include);
      }
      else if ($variables->debug) {
        echo "Failed to find cache include file '$include'.<br />\n";
      }
      $function = "ad_cache_$variables->cache". '_variables';
      if (function_exists($function)) {
        $external_variables = $function();
        foreach ($external_variables as $key => $value) {
          $variables->$key = $value;
        }
      }
    }

    // Set defaults.
    $variables->quantity = $variables->quantity ? $variables->quantity : 1;

    if ($variables->debug) {
      foreach ($variables as $variable => $value) {
        echo "$variable: '$value'<br />\n";
      }
      if ($variables->debug == 1) exit;
    }
    $variables->loaded = TRUE;
  }

  return $variables->$variable;
}

/**
 * Display additional debug information.
 */
function adserve_debug() {
  if (adserve_variable('debug')) {
    echo "Root drupal directory detected as '". adserve_variable('root_dir') ."'.<br />\n<br />\n";

    $ad_dir = adserve_variable('ad_dir');
    $files = array("$ad_dir/adserve.php", "$ad_dir/ad.module");
    if (adserve_variable('debug') > 2) {
      $files = array_merge($files, array("$ad_dir/ad.install", "$ad_dir/ad_views.inc"));
    }
    if (adserve_variable('debug') > 3) {
      $files = array_merge($files, array("$ad_dir/image/ad_image.module", "$ad_dir/image/ad_image.install", "$ad_dir/image/ad_image_views.inc", "$ad_dir/text/ad_text.module", "$ad_dir/text/ad_text.install", "$ad_dir/text/ad_text_views.inc", "$ad_dir/embed/ad_embed.module", "$ad_dir/report/ad_report.module", "$ad_dir/notify/ad_notify.module", "$ad_dir/notify/ad_notify.install"));
    }
    foreach ($files as $file) {
      if (!file_exists($file)) {
        echo "Error: '$file' does not exist!<br />\n";
      }
      else if (!is_readable($file)) {
        echo "Error: '$file' is not readable!<br />\n";
      }
      else {
        $fd = fopen($file, 'r');
        while (!feof($fd)) {
          $line = fgets($fd);
          if (substr($line, 0, 5) == "<?php") {
            continue;
          }
          else {
            echo "$file: $line<br />";
            break;
          }
        }
      }
    }
    echo "<br />\n";
  }
}

/**
 * Include the necessary files and call the Drupal bootstrap.
 */
function adserve_bootstrap($bootstrap = DRUPAL_BOOTSTRAP_FULL) {
  if (adserve_variable('debug')) {
    echo "Drupal bootstrap '". $bootstrap ."'.<br />\n";
  }

  drupal_bootstrap($bootstrap);
}

?>
